// ****************************************************************************
#region Header

// Title       : Card.cs
// Author      : Matthew Tinn
// Date        : 27/03/2013
// Description : Class for a single card

#endregion
// ****************************************************************************
#region UsingDirectives

using UnityEngine;
using System.Collections;

#endregion
// ****************************************************************************
public class Card : System.Object 
{ // The digital card used to hold the data from the database

  // **************************************************************************
  // CLASS MEMBERS

  private int cardID;        // ID of the card
  private int boxID;         // Number of box card is in
  private string front;      // Data on front of card
  private string back;       // Data on back of card
  private bool matched;      // Stores if the card has been matched
  private bool selected;     // Stores if the crad has been selected
  private int sidesSelected; // Works with selected
                             // O = no sides, 1 = 1 side, 2 = both sides
  private int clickCount;    // Number of clicks made on a card
                             // Used for determining how well the use knows a card

  // ****************************************************************************
  // CONSTRUCTORS

  public Card()
  { // Creates a blank card
    // Using InitialiseCard
    InitialiseCard();
  }

  public Card(string front, string back)
  { // Creates a card WITHOUT any identification
    InitialiseCard(front, back);
  }

  public Card(int cardID, int boxID, string front, string back)
  { // Creates a card WITH identification
    InitialiseCard(cardID, boxID, front, back);
  }

  // ****************************************************************************
  // INITIALISATION
  public void NewCard()
  { // Initialises all of the variables using new
    this.cardID = new int();
    this.boxID = new int();
    this.front = new string('a', 1);
    this.back = new string('b', 1);
    this.matched = new bool();
    this.clickCount = 0;
  }

  private void InitialiseCard()
  { // Creates a blank card
    NewCard();
    this.cardID = 0;
    this.boxID = 0;
    this.front = "";
    this.back = "";
    this.matched = false;
    this.selected = false;
  }

  private void InitialiseCard(string front, string back)
  { // Creates a card that isn't identifiable, but has front and back data

    // Create a new card
    NewCard();

    // Initialise the card data
    this.cardID = 0;
    this.boxID = 0;
    this.front = front;
    this.back = back;
    this.matched = false;
    this.selected = false;
  }

  private void InitialiseCard(int cardID, int boxID, string front, string back)
  { // Creates a card with the specified data

    // Create an new card
    NewCard();

    // Intialise the card data
    SetAllCardData(cardID, boxID, front, back);
  }

  // ****************************************************************************
  // GETTERS AND SETTERS

  public int GetCardID()
  { // Returns the card ID
    return this.cardID;
  }

  public void SetCardID(int cardID)
  { // Sets the card ID
    this.cardID = cardID;
  }

  public int GetBoxID()
  { // Returns the box the card is in
    return this.boxID;
  }

  public void SetBoxID(int boxID)
  { // Sets the box the card is in
    this.boxID = boxID;
  }

  public string GetFront()
  { // returns the data on the front of the card
    return this.front;
  }

  public void setFront(string front)
  { // Sets the front of the card
    this.front = front;
  }

  public string GetBack()
  { // returns the data on the back of the card
    return this.back;
  }

  public void setBack(string back)
  { // Sets the back of the card
    this.back = back;
  }

  public bool GetMatched()
  { // Returns matched
    return this.matched;
  }

  public void SetAllCardData(int cardID, int boxID, string front, string back)
  { // Sets all card data to what is passed in to the subprogram
    this.cardID = cardID;
    this.boxID = boxID;
    this.front = front;
    this.back = back;
    this.matched = false;
  }

  public void ChangeBoxID(int newBoxID)
  { // Inserts the card into a different box
    this.boxID = newBoxID;
  }

  // ****************************************************************************
  // MATCHING CARD SUBPROGRAMS

  public void SetMatched()
  { // Card has been matched
    this.matched = true;
  }

  public void SetUnmatched()
  { // Card has not been matched
    this.matched = false;
  }

  // ****************************************************************************
  // SELECTED CARD SUBPROGRAMS

  public void setSelected()
  { // Sets the card has been selected
    if(IncrementSidesSelected())
    {
      this.selected = true;
    }
  }

  public void setUnselected()
  { // Sets that the card has not been selected,
    // or deselected
    this.DecrementSidesSelected(); // Decrement sides selected
    this.selected = false;
  }

  public bool GetSelected()
  { // Returns whether the card has been selected
    return this.selected;
  }

  private bool IncrementSidesSelected()
  { // Sets the number of sides selected for a card

    if (this.sidesSelected <= 1)
    { // If a side has not been selected
      // Increment the number of sides and return true
      this.sidesSelected++;
      return true;
    }
    else
    { // Otherwise note error that the number of sides selected has already
      // at its maximum amount
      Debug.Log("Error: Tried to increment sidesSelected when sidesSelected is " + sidesSelected);
    }

    // Return false if sides were not incremented.
    return false;
  }

  private bool DecrementSidesSelected()
  { // Decrements sides selected

    if (this.sidesSelected >= 1)
    { // If there sides selected,
      // decremented the value
      this.sidesSelected--;
      return true;
    }
    else
    { // If zero, log error and do nothing
      Debug.Log("Error: Tried to decrement sidesSelected when sidesSelected is " + sidesSelected);
    }

    // Return false if sides were not decremented
    return false;
  }

  private bool ClearAllSidesSelected()
  { // Sets sides selected to zero
    this.sidesSelected = 0;
    return true;
  }

  public bool IncrementClickCount()
  { // Called when a card is clicked
    // Increments number of times the card has been clicked
    this.clickCount++;
    Debug.Log("Click count incremeneted. Current: " + this.clickCount);
    return true;
  }

  public void resetClickCount()
  { // Resets then umber of times a card has been clicked
    this.clickCount = 0;
  }

  public int GetClickCount()
  { // Returns the number of times a card has been clicked
    return clickCount;
  }
} // End of class
