﻿// ****************************************************************************
#region header

// Title       : CharacterSetManager.cs
// Author      : Matthew Tinn
// Date        : 27/03/2013
// Description : Manages the importing of character sets into the character database


#endregion
// ****************************************************************************
#region includes

using UnityEngine;
using System;
using System.IO;

#endregion
// ****************************************************************************
class CharacterSetManager
{ // Manages the importing of character sets into the database

  // ****************************************************************************
  // CLASS MEMBERS
  const int MAX_CHAR_PLUS_READING = 200;  // The maxium size of the input data
                                          // Currently supports 100 characters at once

  // ****************************************************************************
  // CONSTRUCTOR
  public CharacterSetManager()
  {
  }

  // ****************************************************************************
  // IMPORT SUBPORGRAMS

  public void ImportCharacterSet()
  {
    // Imports a character set
    // Simply imports from project folder
    // For expansion would build a file browser to allow the user to select a file.

    // Create an instance of the custom FileIO class
    FileIO characterReader = new FileIO();

    // Create two strings
    // The raw data holds the character direct from the text file
    string[] rawData = new string[MAX_CHAR_PLUS_READING];

    // The character array holds the raw data along with ' ' in order
    // for the string to be understood in the SQL query
    string[] characterArray = new string[MAX_CHAR_PLUS_READING];

    // Read the raw data from the file
    characterReader.ReadFromFile("japan001.chset", ref rawData, MAX_CHAR_PLUS_READING);

    for (int i = 0; i < MAX_CHAR_PLUS_READING; i++)
    { // Convert a raw data into a string that can be understood by SQL
      string newString = "'" + rawData[i] + "'";
      characterArray[i] = newString;
    }

    // Note on file arrangement
    // ODD characters must be FOREIGN
    // EVEN characters must be the NATIVE READING
    // e.g JP/EN

    // Extract the read data into several different arrays
    // Ready for database processing

    int[] cardID = new int[MAX_CHAR_PLUS_READING / 2];
    int[] boxID = new int[MAX_CHAR_PLUS_READING / 2];
    string[] front = new string[MAX_CHAR_PLUS_READING / 2];
    string[] back = new string[MAX_CHAR_PLUS_READING / 2];

    for (int i = 0; i < MAX_CHAR_PLUS_READING / 2; i++)
    { // Set the card IDs for each card
      cardID[i] = i;

      // The box ID is set to the first default box
      // As we start counting most things at zero, we will call
      // the first box box 0.
      boxID[i] = 0;
    }

    for (int i = 0; i < MAX_CHAR_PLUS_READING/2; i++)
    {
      // First off, set front
      if (i == 0)
      { // If i = 0, just copy the zeroth element of the char array
        front[i] = characterArray[i];
      }
      else
      { // Otherwise have to multiply i by factor 2
        // to account for jump in position
        // between front and back of card in raw data
        front[i] = characterArray[i*2];
      }
    }

    int k = 0; // A counting index

    // For back[i] = characterArray[j] to work
    // We use the formula
    // j = i + (k + 1)

    for (int i = 0; i < MAX_CHAR_PLUS_READING / 2; i++)
    { // Copy each back item from the character array into the
      // back array
      int j = 0;

      k++;

      j = i + k;

      back[i] = characterArray[j];
    }

    // ***
    // Debug code to check front and back arrays are correct

    for (int i = 0; i < MAX_CHAR_PLUS_READING / 2; i++)
    {
        Debug.Log("Front" + i + "is" + front[i]);
    }

    for (int i = 0; i < MAX_CHAR_PLUS_READING / 2; i++)
    {
        Debug.Log("Back" + i + "is" + back[i]);
    }
    // ***

    // Create the database connection
    Database dataConnection = new Database("character.db");

    // Crate two arrays for table creation
    // One holding the column names
    // One holding the column data types
    string[] columns = new string[4] { "cardID", "boxID", "front", "back" };
    string[] dataTys = new string[4] { "int", "int", "nvarchar(5)", "varchar(50)"};

    if (!dataConnection.CheckTableExists("character")) // VSHOW
    { // If the table doesn't exist, create the table
      dataConnection.CreateTable("character", columns, dataTys);
    }

    // Insert the character data into the database
    for (int i = 0; i < MAX_CHAR_PLUS_READING / 2; i++)
    {
      // Store each card in a values string array
      // That is used to build the SQL query in the database code
      string[] values = new string[4];

      values[0] = cardID[i].ToString();
      values[1] = boxID[i].ToString();
      values[2] = front[i];
      values[3] = back[i];

      Debug.Log("Before Insert into front is " + front[i] + " and values[2] is " + values[2]);

      // Pass to insert into, which will insert the data into the database
      //dataConnection.InsertInto("character", values);
      dataConnection.InsertIntoSpecific("character", columns, values);

      Debug.Log("End insert");
    }

    // When done, close the connection to the database
    dataConnection.CloseConnection();
  }
}// End of class