// ****************************************************************************
#region header

// Title       : DeckManager.cs
// Author      : Matthew Tinn
// Date        : 23/05/2013
// Description : Manages the current working deck of cards in the game

#endregion
// ****************************************************************************
#region includes

using UnityEngine;
using System;
using System.IO;
using Mono.Data.Sqlite;

#endregion
// ****************************************************************************
class DeckManager
{ // **************************************************************************
  // VARIABLES
  private int masterDataEntryIndex;   // 
  private bool dataNotFound;          // Indicates if data was not found
  private bool currentIndexStored;    // 

  // **************************************************************************
  // CONSTRUCTORS
  public DeckManager()
  {
    // Set member variables to defaults
    masterDataEntryIndex = 0;

    dataNotFound = true;
    currentIndexStored = false;
  }

  public int GetColumnCount(string tableName, string[] column)
  { // Retrives the column count for tableName

    // Build the SQL query
    string[] items = new string[1];

    items[0] = "COUNT(" + column[0] + ")";

    // Create a new database connection
    Database dataConnection = new Database("character.db");

    // Create a new SQLite Data Reader, executing the query
    SqliteDataReader countReader = dataConnection.SelectFrom(tableName, items);

    // Read the SELECT COUNT data from the data reader 
    countReader.Read();

    int response; // Variable to store the data retrieved as an int

    // Get the response as an integer
    response = countReader.GetInt32(0);

    // Close the data reader and database connection
    countReader.Close();
    countReader = null;

    dataConnection.CloseConnection();
    dataConnection = null;

    Debug.Log("The column count is " + response + " for column " + column[0]);

    return response;
  }

  public void DebugColumnExists()
  { // DEBUG SUBPROGRAM
    // Checks that the column exists

    // Outputs to the console the count of each column
    // in the database
    string tableName = "character";

    string[] column = new string[1] { "boxID" };

    int countIDs = -1;

    countIDs = this.GetColumnCount(tableName, column);
    Debug.Log("Successfully returned: " + countIDs);

    countIDs = -1;

    column[0] = "cardID";
    countIDs = this.GetColumnCount(tableName, column);
    Debug.Log("Successfully returned: " + countIDs);

    countIDs = -1;

    column[0] = "front";
    countIDs = this.GetColumnCount(tableName, column);
    Debug.Log("Successfully returned: " + countIDs);

    countIDs = -1;

    column[0] = "back";
    countIDs = this.GetColumnCount(tableName, column);
    Debug.Log("Successfully returned: " + countIDs);
  }

  public void NewRetrieveCardData(ref Card[] dataArray, int dataSize)
  { // Retrieves card data from the database

    // Set the tablename
    string tableName = "character";

    // Set the SQL statement strings
    string[] items = new string[1] {"COUNT (DISTINCT cardID)"};
    string[] column = new string[1] {"boxID"};
    string[] columns = new string[2] {"boxID", "cardID"};
    string[] operation = new string[1] {"="};
    string[] operations = new string[2] {"=", "="};
    string[] value = new string[1] {"0"};
    string[] values = new string[2] {"0", "0"};

    // Create temp data arrays
    int[] cardIDArray = new int[dataSize];
    int[] boxIDArray = new int[dataSize];
    string[] frontArray = new string[dataSize];
    string[] backArray = new string[dataSize];

    int cardBoxCount = 0;    // The number of card boxes in the Leitner System
    int numCardsAdded = 0;   // The number of cards added to the deck

    for (int i = 0; i < 5; i++)
    { // Nested loop

      // First retrieve box counts for each box, i
      // Then store the correct number of cards, n
      // until n is 25.

      if (i > 0)
      { // Build the SQL query string for box i
        items[0] = "COUNT (DISTINCT cardID)";
        column[0] = "boxID";
        columns[0] = "boxID";
        columns[1] = "cardID";
        operation[0] = "=";
        operations[0] = "=";
        operations[1] = "=";
        value[0] = "i";
        values[0] = "i";
        values[1] = "i";
      }

      // Set the box numbers in the array
      SetArrayBoxNumbers(ref values, i);

      value[0] = i.ToString();

      // Create a database connection
      Database dataConnection = new Database("character.db");

      // Get the amount of cards in the box
      SqliteDataReader getCardBoxCount = dataConnection.SelectFromWhere(tableName, items, column, operation, value, true);

      getCardBoxCount.Read(); // Read the data

      if (getCardBoxCount.IsDBNull(0) == false)
      { // If the SQl query did not return null
        // set the cardBoxCount
        Debug.Log(i);
        cardBoxCount = getCardBoxCount.GetInt32(0);
      }

      // Close the data reader
      getCardBoxCount.Close();
      getCardBoxCount = null;

      // Close database connection
      dataConnection.CloseConnection();
      dataConnection = null;

      // Output the number of cards in box i to the console
      Debug.Log("The amount of cards in box " + i + " is " + cardBoxCount);

      for (int n = 0; n < 25; n++)
      { // Now for each box get a selection of cards until
        // the box is exhausted of cards or
        // the number of cards retrived is 25 

        // When a box has all of its cards retrieved, go to next box

        if (cardBoxCount >= 5 && numCardsAdded < dataSize)
        {
          // Set the correct SQL statement strings
          values[0] = i.ToString();
          values[1] = n.ToString();

          if (!currentIndexStored)
          { // If the current number of cards has not been stored
            // Set the master number of cards to be stored
            // as n
            masterDataEntryIndex = n;
            currentIndexStored = true;
          }

          if(!dataNotFound)
          { // If no data was returnd, increment master data entry index
            masterDataEntryIndex++;
          }

          // Want to retrive the cardID
          items[0] = "cardID";

          // Build the array of cardIDs
          Debug.Log("n is " + n);
          cardIDArray[masterDataEntryIndex] = GetIntTypeData(tableName, items, columns, operations, values, n);

          if (!dataNotFound && n < 25)
          { // If data is present and the number of cards currently stored is < 25
            // Retrieve the next set of cards from the next box
            Debug.Log(n);
            Debug.Log("Data: " + cardIDArray[masterDataEntryIndex]);

            items[0] = "boxID";

            //select boxID from character where boxID = i and cardID = n
            boxIDArray[masterDataEntryIndex] = GetIntTypeData(tableName, items, columns, operations, values, n);
        
            items[0] = "front";
            frontArray[masterDataEntryIndex] = GetStringTypeData(tableName, items, columns, operations, values);
            Debug.Log("Data: " + frontArray[masterDataEntryIndex]);

            items[0] = "back";
            backArray[masterDataEntryIndex] = GetStringTypeData(tableName, items, columns, operations, values);

            numCardsAdded++;
          } // end if
        } // end if
      } // end for
    } // end box loop

    // Array loop
    // Add all of the seperate arrays to the data array
    for (int i = 0; i < dataSize; i++)
    {
      if (cardIDArray[i] != null)
      {
        dataArray[i].SetCardID(cardIDArray[i]);
      }

      if (boxIDArray[i] != null)
      {
        dataArray[i].SetBoxID(boxIDArray[i]);
      }

      if (frontArray[i] != null)
      {
        dataArray[i].setFront(frontArray[i]);
      }

      if (backArray[i] != null)
      {
        dataArray[i].setBack(backArray[i]);
      }

      Debug.Log("Array loop: " + i);
      Debug.Log(dataArray[i].GetCardID());
      Debug.Log(dataArray[i].GetBoxID());
      Debug.Log(dataArray[i].GetFront());
      Debug.Log(dataArray[i].GetBack());
    }
  }

  private int GetIntTypeData(string tableName, string[] items, string[] columns, string[] operations, string[] values, int rowIndex)
  { // Returns read data as an integer
    int item = 0; // Datum to return

    // Create a database connection
    Database dataConnection = new Database("character.db");

    // Create an SQLite Data Reader and perform a select from where and statement
    // using the passed parameters
    SqliteDataReader readData = dataConnection.SelectFromWhereAnd(tableName, items, columns, operations, values);

    // Read the data
    readData.Read();

    if (!readData.IsDBNull(0))
    { // If data is not null
      // store the read data as the item to return
      item = readData.GetInt32(0);

      dataNotFound = false;
    }
    else
    {
      dataNotFound = true;
    }

    // Close the data reader and the database connection
    readData.Close();
    readData = null;

    dataConnection.CloseConnection();
    dataConnection = null;

    return item;
  }

  private string GetStringTypeData(string tableName, string[] items, string[] columns, string[] operations, string[] values)
  { // Returns read data as a string

      string item = ""; // Item to return

      // Create a database connection
      Database dataConnection = new Database("character.db");

      // Create an SQLite Data Reader
      SqliteDataReader readData = dataConnection.SelectFromWhereAnd(tableName, items, columns, operations, values);

      // Read the data
      readData.Read();

      if (!readData.IsDBNull(0))
      { // If the read data is not null
        // Store the read data as the item to return
        item = readData.GetString(0);
      }

      // Close the data reader and the database connection
      readData.Close();
      readData = null;

      dataConnection.CloseConnection();
      dataConnection = null;

      return item;
  }

  private void SetArrayBoxNumbers(ref string[] values, int i)
  { // Sets the values string box numbers based on the value of i
    // for the SQL query string
    if (i == 1)
    {
      values[0] = "1";
    }
    else if (i == 2)
    {
      values[0] = "2";
    }
    else if (i == 3)
    {
      values[0] = "3";
    }
    else if (i == 4)
    {
      values[0] = "4";
    }
  }

  public void UpdateCardData(Card[] cardArray, int arraySize, string tableName)
  { // Updates the card data in the database

    Card[] updateArray = new Card[arraySize]; // the array of cards to update

    int numCardsToUpdate = 0; // The number of cards to update

    for (int i = 0; i < arraySize; i++)
    { // Initalise the update array
      updateArray[i] = new Card();
    }

    // Loop through cards, checking their click count
    for (int i = 0; i < arraySize; i++)
    {
      // Build an array of cards that only have their click count
      // as 2 - once for front, once for back

      // Copies all other data from the original card
      if (cardArray[i].GetClickCount() == 2)
      {
        updateArray[i].SetCardID(cardArray[i].GetCardID());

        if (cardArray[i].GetBoxID() < 5)
        { // Add 1 here to increment the boxID
          updateArray[i].SetBoxID(cardArray[i].GetBoxID() + 1);
        }

        updateArray[i].setFront(cardArray[i].GetFront());
        updateArray[i].setBack(cardArray[i].GetBack());

        numCardsToUpdate++;

        // We don't care about the other value for the database
        // update, so we can leave them alone
      }
    }
    // These cards are moved into the next box

    // Cards that have had more than 2 clicks stay the same

    // Now we build the SQL update statement

    // Statement we want

    // UPDATE tablename Set column operation function WHERE condColumns[] condOper condValues[] AND ...
    // UPDATE character Set boxID = updateArray[i].boxID() WHERE cardID = updateArray[i].cardID()

    string[] updtColumns = new string[1] {"boxID"};
    string[] updtOper = new string[1] {"="};
    string updtFunction;
    string[] condColumns = new string[1] {"cardID"};
    string[] condOpers = new string[1] {"="};
    string[] condValues = new string[1];

    // Main update loop
    // Want to update one datum at a time
    // To prvent data corruption
    // i.e. Every tuple in characters being affected by the statement

    for (int i = 0; i < numCardsToUpdate; i++)
    {
      updtFunction = updateArray[i].GetBoxID().ToString();
      condValues[0] = updateArray[i].GetCardID().ToString();

      // Create a database connection
      Database updateConnection = new Database("character.db");

      // Create an SQLite Data Reader and perform the update
      SqliteDataReader updater = updateConnection.UpdateWhereOnce(tableName, updtColumns, updtOper, updtFunction,
                                                                  condColumns, condOpers, condValues);

      // Close the upater
      updater.Close();
      updater = null;

      // Check data has gone in

      // SELECT cardId from character wherer boxid is cardArray[i].boxid
      string[] items = new string[1] {"cardID"};
      string[] column = new string[1] {"boxID"};
      string[] operation = new string[1] {"="};
      string[] values = new string[1] {updateArray[i].GetBoxID().ToString()};

      // Creater a checker data reader and perform the query
      SqliteDataReader checker = updateConnection.SelectFromWhere(tableName, items, column, operation, values, true);

      // Read the data
      checker.Read();
      int returnData = checker.GetInt32(0);

      // Close the data reader and the database connection
      checker.Close();
      checker = null;

      updateConnection.CloseConnection();
    }
  }
}