﻿// ****************************************************************************
#region header

// Title       : FileIO.cs
// Author      : Matthew Tinn
// Date        : 27/03/2013
// Description : Custom File IO Class


#endregion
// ****************************************************************************
#region includes

using UnityEngine;
using System;
using System.IO;
using System.Collections;

#endregion
// ****************************************************************************
class FileIO
{ // Opertaes the file input/output for the application

  // ****************************************************************************
  // CONSTRUCTOR
  public FileIO()
  {
  }

  // ****************************************************************************
  // FILE READING
  public bool ReadFromFile(string filename, ref string[] lineArray, int arraySize)
  { // Read from a file
    // passes in filename to access, an array to store each line in and the size of the array

    bool isRead = false; // Specifies if file has been read
    
    // Set the filepath
    string path;
    path = System.IO.Directory.GetCurrentDirectory();
    path += "\\";
    path += filename;

    if (File.Exists(path))
    { // If the file exists
      // Create a streamreader to read the file
      StreamReader fileReader = new StreamReader(path);

      // Read each line, storing it in an array
      // Array returned by reference to the calling subprogram

      int linesRead = 0; // Stores the number of lines read

      try
      {
        while (fileReader.Peek() != -1 && linesRead != arraySize)
        { // Reads each line of the file, storing it in the array
          lineArray[linesRead] = fileReader.ReadLine();
          linesRead++; //Increment lines read
        }

        isRead = true;
      }

      catch
      { // Catch block for empty or non existant file
        Debug.Log("The file is empty or does not exist");
        lineArray[0] = " ";
      }

      finally
      { // Close the file
        fileReader.Close();
      }
    }

    return isRead;
  } // End file reading

  // ****************************************************************************
  // FILE WRITING

  public void WriteToFile(string filename, string lineToWrite)
  { // Writes a single line to a file

    // Build the path to the file
    string path;
    path = System.IO.Directory.GetCurrentDirectory(); // Gets the current project directory
    path += "\\";     // Add a \ character
    path += filename; // Add the filename

    // Create a new StreamWriter object
    StreamWriter fileWriter = new StreamWriter(path);

    // Write the line to the file
    fileWriter.WriteLine(lineToWrite);

    // Close the file
    fileWriter.Close();

    // Log that file has been written to
    Debug.Log("Wrote to file " + path);
  }

  public void WriteToFile(string filename, string[] writeArray, int arraySize)
  { // Write multiple lines (given in an array) to a single file
    
    // Build the path to the file
    string path;
    path = System.IO.Directory.GetCurrentDirectory(); // Get the current project directoy
    path += "\\"; // Add a \ character
    path += filename; // Add the filename

    // Create a new StreamWriter object
    StreamWriter fileWriter = new StreamWriter(path);

    // Loop through the line array, writing each one
    for (int i = 0; i < arraySize; i++)
    {
      fileWriter.WriteLine(writeArray[i]);
    }

    // Close the file when complete
    fileWriter.Close();

    // Log that file has been written to
    Debug.Log("Wrote to file " + path);
  }
}
