// ****************************************************************************
#region Header

// Title       : MenuGUI.cs
// Author      : Matthew Tinn
// Date        : 27/03/2013
// Description : Class for GUI elements of menu screen

#endregion
// ****************************************************************************
#region UsingDirectives

using UnityEngine;
using System.Collections;

#endregion
// ****************************************************************************
public class MenuGUI : MonoBehaviour 
{ // A class for producing the GUI of the main menu
  // **************************************************************************
  // CLASS MEMBERS

  public GUISkin menuSkin;          // GUI skin

  private Rect buttonGame1;         // Rect for the game button 1
  private Rect buttonGame2;         // Rect for the game button 2
  private Rect buttonGame3;         // Rect for the game button 3
  private Rect buttonGame4;         // Rect for the game button 4
  private Rect buttonExit;          // Rect for the exit button 
  private Rect buttonImport;        // Rect for the character import button

  float buttonTop01;                // Locations of the tops of buttons
  float buttonTop02;
  float buttonTop03;
  float buttonTop04;

  float buttonLeft;                 // Left position of buttons

  // Specify button height and width constants
  private const int menuButtonWidth = 200;
  private const int menuButtonHeight = 50;

  private bool sure;                // Determines whether the user want to import new character set

  // **************************************************************************
  // UNITY ENGINE SUBPROGRAMS
  
  // **** Start ***************************************************************
  void Start()
  { // Initialisation

    // Need to define four positions for where the buttons will be placed
    buttonTop01 = Screen.height / 2 - menuButtonHeight - 100;
    buttonTop02 = buttonTop01 + 100;
    buttonTop03 = buttonTop02 + 100;
    buttonTop04 = buttonTop03 + 100;

    // Set the value for the left indentation of the buttons
    buttonLeft = 20;

    // Set the positions of the buttons on the main menu.
    this.buttonGame1.Set(buttonLeft, buttonTop01, menuButtonWidth, menuButtonHeight);
    this.buttonGame2.Set(buttonLeft, buttonTop02, menuButtonWidth, menuButtonHeight);
    this.buttonGame3.Set(buttonLeft, buttonTop03, menuButtonWidth, menuButtonHeight);
    this.buttonGame4.Set(buttonLeft, buttonTop04, menuButtonWidth, menuButtonHeight);

    // Create position for the exit button
    float exitLeft = Screen.width - (menuButtonWidth);
    float exitTop = (float)(Screen.height - (1.5*menuButtonHeight));

    // Set the exit button position
    this.buttonExit.Set(exitLeft, exitTop, menuButtonWidth, menuButtonHeight);

    // Create position for the re-import character set button
    float importLeft = exitLeft;
    float importTop = (float)(Screen.height - (3.5*menuButtonHeight));

    //Set the import button position
    this.buttonImport.Set(importLeft, importTop, menuButtonWidth, menuButtonHeight);
  } // End Start()

  // **** OnGUI ***************************************************************
  void OnGUI()
  { // Manages all GUI code

    // Assign the engine GUI skin to the menuSkin object
    GUI.skin = this.menuSkin;

    // Begin a new GUI layout area
    GUILayout.BeginArea(new Rect(0, 0, Screen.width, Screen.height));

    // Display the game descriptions in GUI boxes
    GUI.Box(new Rect(buttonLeft + menuButtonWidth, buttonTop01, Screen.width / 2, menuButtonHeight), "Match characters with their meanings.");
    GUI.Box(new Rect(buttonLeft + menuButtonWidth, buttonTop02, Screen.width / 2, menuButtonHeight), "Select the correct characters to complete the phrase.");
    GUI.Box(new Rect(buttonLeft + menuButtonWidth, buttonTop03, Screen.width / 2, menuButtonHeight), "Stop the card wheel at the right moment.");
    GUI.Box(new Rect(buttonLeft + menuButtonWidth, buttonTop04, Screen.width / 2, menuButtonHeight), "Connect the characters in the right order.");

    // Display the buttons
    // If a button is clicked, perform an action
    if (GUI.Button(buttonExit, "Exit"))
    { // Exit button quits the application
      this.TestButton(0);
      Application.Quit();
    }

    if (GUI.Button(buttonGame1, "Match-Me!"))
    { // buttonGame1 loads the game 'Match Me'
      this.TestButton(1);
      Application.LoadLevel("MatchMe");
    }

    if (GUI.Button(buttonGame2, "Complete the Phrase"))
    { // buttonGame2 loads the game 'Complete the Phrase'
      this.TestButton(2);
    }

    if (GUI.Button(buttonGame3, "Character Roulette"))
    { // buttonGame3 loads the game 'Character Roulette'
      this.TestButton(3);
    }

    if (GUI.Button(buttonGame4, "Shuffle!"))
    { // buttonGame4 loads the game 'Shuffle'
      this.TestButton(4);
    }

    if (!sure)
    { // If the user has not pressed the 'Sure?' button,
      // show the import character button
      if (GUI.Button(buttonImport, "Character Import"))
      { // When clicked, set the variable that controls the display of the
        // sure button to true
        this.TestButton(5);
        sure = true;
      }
    }
    else if (sure)
    { // If the user needs to press the Sure? button, imports a character
      // set from the database
      if (GUI.Button(buttonImport, "Sure?"))
      {
        // Create a character set manager
        CharacterSetManager charManager = new CharacterSetManager();

        // Importh the character set using the manager
        charManager.ImportCharacterSet();
      }
    }

    // End the GUI layout area
    GUILayout.EndArea();
  } // End OnGUI()

  // ****************************************************************************

  // TESTING SUBPROGRAMS

  void TestButton(int buttonNumber)
  { // Produces debug test code for the pressing of a button

    // The button pressed passes in its number, which is used
    // to identify itself and output to the console that the
    // button has been pressed

    if (buttonNumber == 0)
    {
        Debug.Log("Exit button was pressed.");
    }
    else if (buttonNumber == 1)
    {
        Debug.Log("Button for game 1 was pressed.");
    }
    else if (buttonNumber == 2)
    {
        Debug.Log("Button for game 2 was pressed.");
    }
    else if (buttonNumber == 3)
    {
        Debug.Log("Button for game 3 was pressed.");
    }
    else if (buttonNumber == 4)
    {
        Debug.Log("Button for game 4 was pressed.");
    }
    else if (buttonNumber == 5)
    {
        Debug.Log("Button for character import was pressed.");
    }
    else
    {
        Debug.Log("Button test was called on an unknown or incorrect button number.");
    }
  } // End TestButton
} // End class
